==> spread-5.6.1.59-py3-none-any.whl: spread_lom/interface.py <==
import base64
import json
import logging
import socket
import threading
from time import sleep

import cbor
import paho.mqtt.client

from spread_core.tools.base_interface import AbstractInterfaceClass
from spread_core.tools.utils import bytes2str
from . import commands


# UPLINK = 'airbit/light/ctrl/sample'
# DOWNLINK = 'send/data/'


class LomInterface(AbstractInterfaceClass, threading.Thread):
    wait_for_ready = True

    def __init__(self):
        super(LomInterface, self).__init__(None)
        threading.Thread.__init__(self, name=self.__class__.__name__)

        self._mqttc = None
        self._address = None
        self._auth = None
        self._send_topic = None
        self._dump_topic = None
        self._to_subscribe = []
        self._answer_event = threading.Event()
        self.__response = None
        self.__request = None

    @property
    def _request(self) -> commands.LFrame: return self.__request

    @_request.setter
    def _request(self, request):
        self.__request = request

    @property
    def _response(self) -> commands.LFrame: return self.__response

    @_response.setter
    def _response(self, response): self.__response = response

    def start(self):
        threading.Thread.start(self)

    def run(self):
        self.do_connect()
        self.mqttc.loop_forever()

    def set_bus_params(self, bus: dict):
        self._address = (bus['host'], bus['port'])
        self._auth = (bus['login'], bus['pwd'])
        self._send_topic = bus['send_topic']
        self._dump_topic = bus['dump_topic']

    def send_fast(self, command: commands.LCommand):
        topic = '/'.join([self._send_topic, command.dev_eui])
        logging.debug('>> %s' % command)
        self.publish(topic, command.frame)

    def send(self, command: commands.LCommand, timeout=15, attempts=2):
        _attempts = 0

        try:
            topic = '/'.join([self._send_topic, command.dev_eui])

            if isinstance(command, commands.SetConfig):
                timeout += 20

            frame = command.frame

            self._request = command
            self._response = None

            while self._response is None and _attempts < attempts and not self._stopped:
                if _attempts > 0:
                    sleep(3)
                if not command.is_mc or _attempts == 0:
                    logging.debug('-'*_attempts + '-> %s' % command)
                    self.publish(topic, frame)
                else:
                    logging.debug('%s-> wait %s sec...' % ('-'*_attempts, timeout))

                _attempts += 1

                self._answer_event.wait(timeout)
                self._answer_event.clear()

                if command.is_mc or _attempts == attempts or self._response:
                    yield self._response

                if command.is_mc:
                    self._response = None

            if not command.is_mc and self._response is None:
                logging.debug(f'      X {command}')

            self._request = None
        except Exception as ex:
            logging.error(ex)

    def on_disconnect(self, client, userdata, rc):
        self.do_connect()

    def do_connect(self):
        while not self._stopped:
            logging.info('Try connect to {}:{}'.format(*self._address))
            try:
                self.mqttc.connect(host=self._address[0], port=self._address[1])
            except socket.gaierror as ex:
                logging.error(ex)
            else:
                break

    @property
    def mqttc(self):
        if self._mqttc is None:
            _mqttc = paho.mqtt.client.Client(userdata=self.__class__.__name__, clean_session=True)
            _mqttc.on_connect = self.on_connect
            _mqttc.on_message = self.on_message
            _mqttc.on_disconnect = self.on_disconnect
            _mqttc.on_subscribe = self.on_subscribe
            _mqttc.username_pw_set(*self._auth)
            self._mqttc = _mqttc
        return self._mqttc

    def on_connect(self, mosq, obj, flags, rc):
        connect_results = ['Connection successful',
                           'Connection refused - incorrect protocol version',
                           'Connection refused - invalid client identifier',
                           'Connection refused - server unavailable',
                           'Connection refused - bad username or password',
                           'Connection refused - not authorised']
        if rc < len(connect_results):
            logging.info(connect_results[rc])
            if rc != 0:
                self.ready = False
                self.do_connect()
            else:
                self.subscribe(self._send_topic + '/#')
                self.subscribe(self._dump_topic)
        else:
            logging.warning("rc: " + str(rc))

    def subscribe(self, topic):
        self._to_subscribe.append(topic)
        logging.debug('subscribe to ' + topic)
        self.mqttc.subscribe(topic)

    def on_subscribe(self, client, userdata, mid, granted_qos, properties=None):
        logging.debug('subscribed to ' + self._to_subscribe.pop(0))
        if not self.ready and len(self._to_subscribe) == 0:
            self.ready = True

    def publish(self, topic, data: dict):
        self.mqttc.publish(topic, json.dumps(data))

    def on_message(self, mosq, obj, msg):
        try:
            data = json.loads(msg.payload.decode())
            encoded = base64.b64decode(data['data'])
            decoded = cbor.loads(encoded)
            # logging.debug(f'BASE64: [{msg.topic}]: {data}')
            # logging.debug(f'  CBOR: [{msg.topic}]: {encoded}')
            # logging.debug(f'  DATA: [{msg.topic}]: {decoded}')

            dev_eui = data['mcaddr'] if 'mcaddr' in data else data['dev_eui']
            fport = data['fport']

            if msg.topic == self._dump_topic:
                if fport == 1:
                    response = commands.response_of(dev_eui, decoded)

                    logging.debug(f'    <- {response} {data}')

                    if response.valid:
                        if self._request and self._request.check_response(response):
                            self._response = response
                            self._answer_event.set()
                        else:
                            self.event_handler.send(response)

                elif fport == 2:
                    event = commands.event_of(dev_eui, decoded)
                    logging.debug(f'<- {event}')
                    self.event_handler.send(event)
                else:
                    logging.debug(f'    <- {dev_eui}(fport {fport}): [{bytes2str(encoded)}]')
            elif fport == 1:
                command = commands.request_of(dev_eui, decoded)
                logging.debug(f'  -> ✓: {command} {data}')
                self.external_cmd_handler.send((command,))
            else:
                logging.debug(f'  -> ✓ {dev_eui}(fport {fport}): [{bytes2str(encoded)}]')

        except BaseException as ex:
            logging.warning(f'[{msg.topic}]: {msg.payload}')
            logging.exception(ex)

    def set_stopped(self):
        super(LomInterface, self).set_stopped()
        if self.mqttc:
            try:
                self.mqttc.disconnect()
                self.mqttc.loop_stop(True)
            except BaseException as ex:
                logging.error(ex)

        if not self._answer_event.is_set():
            self._answer_event.set()

==> spread-5.9.0.59-py3-none-any.whl: spread_lom/interface.py <==
import base64
import json
import logging
import socket
import threading
from time import sleep

import cbor
import paho.mqtt.client

from spread_core.tools.base_interface import AbstractInterfaceClass
from spread_core.tools.utils import bytes2str
from . import commands


# UPLINK = 'airbit/light/ctrl/sample'
# DOWNLINK = 'send/data/'


class LomInterface(AbstractInterfaceClass, threading.Thread):
    wait_for_ready = True

    def __init__(self):
        super(LomInterface, self).__init__(None)
        threading.Thread.__init__(self, name=self.__class__.__name__)

        self._mqttc = None
        self._address = None
        self._auth = None
        self._send_topic = None
        self._dump_topic = None
        self._to_subscribe = []
        self._answer_event = threading.Event()
        self.__response = None
        self.__request = None

    @property
    def _request(self) -> commands.LFrame: return self.__request

    @_request.setter
    def _request(self, request):
        self.__request = request

    @property
    def _response(self) -> commands.LFrame: return self.__response

    @_response.setter
    def _response(self, response): self.__response = response

    def start(self):
        threading.Thread.start(self)

    def run(self):
        self.do_connect()
        self.mqttc.loop_forever()

    def set_bus_params(self, bus: dict):
        self._address = (bus['host'], bus['port'])
        self._auth = (bus['login'], bus['pwd'])
        self._send_topic = bus['send_topic']
        self._dump_topic = bus['dump_topic']

    def send_fast(self, command: commands.LCommand):
        topic = '/'.join([self._send_topic, command.dev_eui])
        logging.debug('>> %s' % command)
        self.publish(topic, command.frame)

    def send(self, command: commands.LCommand, timeout=15, attempts=2):
        _attempts = 0

        try:
            topic = '/'.join([self._send_topic, command.dev_eui])

            if isinstance(command, commands.SetConfig):
                timeout += 20

            frame = command.frame

            self._request = command
            self._response = None

            while self._response is None and _attempts < attempts and not self._stopped:
                if _attempts > 0:
                    sleep(3)
                if not command.is_mc or _attempts == 0:
                    logging.debug('-'*_attempts + '-> %s' % command)
                    self.publish(topic, frame)
                else:
                    logging.debug('%s-> wait %s sec...' % ('-'*_attempts, timeout))

                _attempts += 1

                self._answer_event.wait(timeout)
                self._answer_event.clear()

                if command.is_mc or _attempts == attempts or self._response:
                    yield self._response

                if command.is_mc:
                    self._response = None

            if not command.is_mc and self._response is None:
                logging.debug(f'      X {command}')

            self._request = None
        except Exception as ex:
            logging.error(ex)

    def on_disconnect(self, client, userdata, rc):
        self.do_connect()

    def do_connect(self):
        while not self._stopped:
            logging.info('Try connect to {}:{}'.format(*self._address))
            try:
                self.mqttc.connect(host=self._address[0], port=self._address[1])
            except socket.gaierror as ex:
                logging.error(ex)
            else:
                break

    @property
    def mqttc(self):
        if self._mqttc is None:
            _mqttc = paho.mqtt.client.Client(userdata=self.__class__.__name__, clean_session=True)
            _mqttc.on_connect = self.on_connect
            _mqttc.on_message = self.on_message
            _mqttc.on_disconnect = self.on_disconnect
            _mqttc.on_subscribe = self.on_subscribe
            _mqttc.username_pw_set(*self._auth)
            self._mqttc = _mqttc
        return self._mqttc

    def on_connect(self, mosq, obj, flags, rc):
        connect_results = ['Connection successful',
                           'Connection refused - incorrect protocol version',
                           'Connection refused - invalid client identifier',
                           'Connection refused - server unavailable',
                           'Connection refused - bad username or password',
                           'Connection refused - not authorised']
        if rc < len(connect_results):
            logging.info(connect_results[rc])
            if rc != 0:
                self.ready = False
                self.do_connect()
            else:
                self.subscribe(self._send_topic + '/#')
                self.subscribe(self._dump_topic + '/#')
        else:
            logging.warning("rc: " + str(rc))

    def subscribe(self, topic):
        self._to_subscribe.append(topic)
        logging.debug('subscribe to ' + topic)
        self.mqttc.subscribe(topic)

    def on_subscribe(self, client, userdata, mid, granted_qos, properties=None):
        logging.debug('subscribed to ' + self._to_subscribe.pop(0))
        if not self.ready and len(self._to_subscribe) == 0:
            self.ready = True

    def publish(self, topic, data: dict):
        self.mqttc.publish(topic, json.dumps(data))

    def on_message(self, mosq, obj, msg):
        try:
            data = json.loads(msg.payload.decode())
            encoded = base64.b64decode(data['data'])
            decoded = cbor.loads(encoded)
            # logging.debug(f'BASE64: [{msg.topic}]: {data}')
            # logging.debug(f'  CBOR: [{msg.topic}]: {encoded}')
            # logging.debug(f'  DATA: [{msg.topic}]: {decoded}')

            dev_eui = data['mcaddr'] if 'mcaddr' in data else data['dev_eui']
            fport = data['fport']

            if msg.topic.startswith(self._dump_topic):
                if fport == 1:
                    response = commands.response_of(dev_eui, decoded)

                    logging.debug(f'    <- {response} {data}')

                    if response.valid:
                        if self._request and self._request.check_response(response):
                            self._response = response
                            self._answer_event.set()
                        else:
                            self.event_handler.send(response)

                elif fport == 2:
                    event = commands.event_of(dev_eui, decoded)
                    logging.debug(f'<- {event}')
                    self.event_handler.send(event)
                else:
                    logging.debug(f'    <- {dev_eui}(fport {fport}): [{bytes2str(encoded)}]')
            elif fport == 1:
                command = commands.request_of(dev_eui, decoded)
                logging.debug(f'  -> ✓: {command} {data}')
                self.external_cmd_handler.send((command,))
            else:
                logging.debug(f'  -> ✓ {dev_eui}(fport {fport}): [{bytes2str(encoded)}]')

        except BaseException as ex:
            logging.warning(f'[{msg.topic}]: {msg.payload}')
            logging.exception(ex)

    def set_stopped(self):
        super(LomInterface, self).set_stopped()
        if self.mqttc:
            try:
                self.mqttc.disconnect()
                self.mqttc.loop_stop(True)
            except BaseException as ex:
                logging.error(ex)

        if not self._answer_event.is_set():
            self._answer_event.set()
